<?php

	class SamlResponse{
		
		private $xml;
		
		private $acsUrl;
		
		private $issuer;
		
		private $audience;
		
		private $username;
		
		private $attributesList;
		
		private $requestID;

		private $subject;
		
		function __construct($username, $acs_url, $issuer, $audience, $attributesList, $requestID){
			$this->xml = new DOMDocument("1.0", "utf-8");
			$this->acsUrl = $acs_url;		
			$this->issuer = $issuer;		
			$this->audience = $audience;
			$this->username = $username;
			$this->attributesList = $attributesList;
			$this->requestID=$requestID;
		}
		
		function createSamlResponse(){
			
			$response_params = $this->getResponseParams();
			
			//Create Response Element
			$resp = $this->createResponseElement($response_params);
			$this->xml->appendChild($resp);
			
			//Build Issuer
			$issuer = $this->buildIssuer();
			$resp->appendChild($issuer);
			
			//Build Status
			$status = $this->buildStatus();
			$resp->appendChild($status);
			$statusCode = $this->buildStatusCode();
			$status->appendChild($statusCode);
			
			//Build Assertion
			$assertion = $this->buildAssertion($response_params);
			$resp->appendChild($assertion);
			$assertion->appendChild($this->xml->createElement('saml:Issuer',$response_params['issuer']));

			$private_key = getcwd() . DIRECTORY_SEPARATOR . 'includes' . DIRECTORY_SEPARATOR . 'resources' . DIRECTORY_SEPARATOR . 'idp-signing.key';
			$this->signNode($private_key, $assertion, $this->subject, $response_params);
						
			$samlResponse = $this->xml->saveXML();
			$samlResponse = str_replace('<?xml version="1.0"?>','',$samlResponse);
			$samlResponse = base64_encode(stripslashes($samlResponse));
			
			return $samlResponse;								
			
		}
		
		function getResponseParams(){
			$response_params = array();
			$time = time();
			$response_params['IssueInstant'] = str_replace('+00:00','Z',gmdate("c",$time));
			$response_params['NotOnOrAfter'] = str_replace('+00:00','Z',gmdate("c",$time+300));
			$response_params['NotBefore'] = str_replace('+00:00','Z',gmdate("c",$time-30));
			$response_params['AuthnInstant'] = str_replace('+00:00','Z',gmdate("c",$time-120));
			$response_params['SessionNotOnOrAfter'] = str_replace('+00:00','Z',gmdate("c",$time+3600*8));
			$response_params['ID'] = $this->generateUniqueID(40);
			$response_params['assertID'] = $this->generateUniqueID(40);
			$response_params['issuer'] = $this->issuer;
			$response_params['Attributes'] = $this->attributesList;
			$public_key = getcwd() . DIRECTORY_SEPARATOR . 'includes' . DIRECTORY_SEPARATOR . 'resources' . DIRECTORY_SEPARATOR . 'idp-signing.crt';
			$objKey = new XMLSecurityKey(XMLSecurityKey::RSA_SHA256,array( 'type' => 'public'));
			$objKey->loadKey($public_key, TRUE,TRUE);
			$response_params['x509'] = $objKey->getX509Certificate();
			return $response_params;
		}
		
		function createResponseElement($response_params){
			$resp = $this->xml->createElementNS('urn:oasis:names:tc:SAML:2.0:protocol','samlp:Response');
			$resp->setAttribute('ID',$response_params['ID']);
			$resp->setAttribute('Version','2.0');
			$resp->setAttribute('IssueInstant',$response_params['IssueInstant']);
			$resp->setAttribute('Destination',$this->acsUrl);
			if(!is_null($this->requestID))
				$resp->setAttribute('InResponseTo',$this->requestID);
			return $resp;
		}
		
		function buildIssuer(){
			$issuer = $this->xml->createElementNS('urn:oasis:names:tc:SAML:2.0:assertion','samlp:Issuer',$this->issuer);
			return $issuer;
		}
		
		function buildStatus(){
			$status = $this->xml->createElementNS('urn:oasis:names:tc:SAML:2.0:protocol','samlp:Status');
			return $status;
		}
		
		function buildStatusCode(){
			$statusCode = $this->xml->createElementNS('urn:oasis:names:tc:SAML:2.0:protocol','samlp:StatusCode');
			$statusCode->setAttribute('Value', 'urn:oasis:names:tc:SAML:2.0:status:Success');
			return $statusCode;
		}
		
		function buildAssertion($response_params){
			$assertion = $this->xml->createElementNS('urn:oasis:names:tc:SAML:2.0:assertion','saml:Assertion');
			$assertion->setAttributeNS('http://www.w3.org/2000/xmlns/', 'xmlns:saml', 'urn:oasis:names:tc:SAML:2.0:assertion');
			$assertion->setAttribute('ID',$response_params['assertID']);
			$assertion->setAttribute('IssueInstant',$response_params['IssueInstant']);
			$assertion->setAttribute('Version','2.0');
			
			//Build Subject
			$subject = $this->buildSubject($response_params);
			$this->subject = $subject;
			$assertion->appendChild($subject);
			
			//Build Condition
			$condition = $this->buildCondition($response_params);
			$assertion->appendChild($condition);
			
			//Build AuthnStatement
			$authnstat = $this->buildAuthnStatement($response_params);
			$assertion->appendChild($authnstat);
			
			//Build Attributes
			$attributes = $response_params['Attributes'];
            if(!empty($attributes)) {
                $attrStatement = $this->buildAttrStatement($response_params);
				$assertion->appendChild($attrStatement);
            }
			
			/*$private_key = <<<RSA_PRIVATE_KEY
-----BEGIN CERTIFICATE-----
MIIC+jCCAmOgAwIBAgIJANqkdbAqN9caMA0GCSqGSIb3DQEBBQUAMIGVMQswCQYD
VQQGEwJJTjELMAkGA1UECAwCTUgxDTALBgNVBAcMBFBVTkUxEzARBgNVBAoMCm1p
bmlPcmFuZ2UxEzARBgNVBAsMCm1pbmlPcmFuZ2UxEzARBgNVBAMMCm1pbmlPcmFu
Z2UxKzApBgkqhkiG9w0BCQEWHGFkbWluaXN0cmF0b3JAbWluaW9yYW5nZS5jb20w
HhcNMTQwNzE1MDc1MjQzWhcNMTQwODE0MDc1MjQzWjCBlTELMAkGA1UEBhMCSU4x
CzAJBgNVBAgMAk1IMQ0wCwYDVQQHDARQVU5FMRMwEQYDVQQKDAptaW5pT3Jhbmdl
MRMwEQYDVQQLDAptaW5pT3JhbmdlMRMwEQYDVQQDDAptaW5pT3JhbmdlMSswKQYJ
KoZIhvcNAQkBFhxhZG1pbmlzdHJhdG9yQG1pbmlvcmFuZ2UuY29tMIGfMA0GCSqG
SIb3DQEBAQUAA4GNADCBiQKBgQDa+WjFRNylcjQ3OxaDnJ8rJ9C2pqxxdVWYKCTo
Fv/bxU4DTx/TpS8w3uqg8aqmlIfxoWRXO3MQSbfP3YHYuap8PLtLuV6aNIf6vG+s
rdg1RuhgaMvbysUXA/k4EWK40V0ZWZbzlJ3fN1LWoSgaTavh9xtm1tSxVb2ZcnCx
RzdNQwIDAQABo1AwTjAdBgNVHQ4EFgQUzF7PbmGiLgY+08Gqtw1xg8OxuFMwHwYD
VR0jBBgwFoAUzF7PbmGiLgY+08Gqtw1xg8OxuFMwDAYDVR0TBAUwAwEB/zANBgkq
hkiG9w0BAQUFAAOBgQBLlFW3wZs9uAXRuRdhMxWcgD+uETCPtL1h8rFa6lr4iZHj
vG1AOe+neVEhz0FnL96jIbmjFymftfiCWmqyh26JcOqXj6f7aFYcrBLL9CvuoQ14
nIwjAz0EXC4uqRgIQoEv9YEK375wGiKKt7WzvOSGTuum//g4ZKf5RIVKY0Bkaw==
-----END CERTIFICATE-----
RSA_PRIVATE_KEY;
			
			
			$this->signAssertion($private_key, $assertion, $subject);*/
			
			return $assertion;
		}
		
		function buildSubject($response_params){
			$subject = $this->xml->createElement('saml:Subject');
			$nameid = $this->buildNameIdentifier();
			$subject->appendChild($nameid);
			$confirmation = $this->buildSubjectConfirmation($response_params);
			$subject->appendChild($confirmation);
			return $subject;
		}
		
		function buildNameIdentifier(){
			$nameid = $this->xml->createElement('saml:NameID',$this->username);
			$nameid->setAttribute('Format','urn:oasis:names:tc:SAML:2.0:nameid-format:email');
			$nameid->setAttribute('SPNameQualifier',$this->audience);
			return $nameid;
		}
		
		function buildSubjectConfirmation($response_params){
			$confirmation = $this->xml->createElement('saml:SubjectConfirmation');
			$confirmation->setAttribute('Method','urn:oasis:names:tc:SAML:2.0:cm:bearer');
			$confirmationdata = $this->getSubjectConfirmationData($response_params);
			$confirmation->appendChild($confirmationdata);
			return $confirmation;
		}
		
		function getSubjectConfirmationData($response_params){
			$confirmationdata = $this->xml->createElement('saml:SubjectConfirmationData');
			$confirmationdata->setAttribute('NotOnOrAfter',$response_params['NotOnOrAfter']);
			$confirmationdata->setAttribute('Recipient',$this->acsUrl);
			if(!is_null($this->requestID))
				$confirmationdata->setAttribute('InResponseTo',$this->requestID);
			return $confirmationdata;
		}
		
		function buildCondition($response_params){
			$condition = $this->xml->createElement('saml:Conditions');
			$condition->setAttribute('NotBefore',$response_params['NotBefore']);
			$condition->setAttribute('NotOnOrAfter',$response_params['NotOnOrAfter']);
			
			//Build AudienceRestriction
			$audiencer = $this->buildAudienceRestriction();
			$condition->appendChild($audiencer);
			
			return $condition;
		}
		
		function buildAudienceRestriction(){
			$audiencer = $this->xml->createElement('saml:AudienceRestriction');
			$audience = $this->xml->createElement('saml:Audience',$this->audience);
			$audiencer->appendChild($audience);
			return $audiencer;
		}
		
		function buildAuthnStatement($response_params){
			$authnstat = $this->xml->createElement('saml:AuthnStatement');
			$authnstat->setAttribute('AuthnInstant',$response_params['AuthnInstant']);
			$authnstat->setAttribute('SessionIndex','_'.$this->generateUniqueID(30));
			$authnstat->setAttribute('SessionNotOnOrAfter',$response_params['SessionNotOnOrAfter']);
			
			$authncontext = $this->xml->createElement('saml:AuthnContext');
			$authncontext_ref = $this->xml->createElement('saml:AuthnContextClassRef','urn:oasis:names:tc:SAML:2.0:ac:classes:Password');
			$authncontext->appendChild($authncontext_ref);
			$authnstat->appendChild($authncontext);
			
			return $authnstat;
		}
		
		function signAssertion($private_key, $assertion, $subject){
			//Private KEY	
			$objKey = new XMLSecurityKey(XMLSecurityKey::RSA_SHA1, array('type' => 'private'));
			$objKey->loadKey($private_key);
						
			//Sign the Assertion
			$objXMLSecDSig = new XMLSecurityDSig();
			$objXMLSecDSig->setCanonicalMethod(XMLSecurityDSig::EXC_C14N);
			$objXMLSecDSig->addReferenceList(array($assertion), XMLSecurityDSig::SHA1,
				array('http://www.w3.org/2000/09/xmldsig#enveloped-signature', XMLSecurityDSig::EXC_C14N),array('id_name'=>'ID','overwrite'=>false));
			$objXMLSecDSig->sign($objKey);
			$objXMLSecDSig->add509Cert($response_params['x509']);
			$objXMLSecDSig->insertSignature($assertion,$subject);
		}
		
		function buildAttrStatement($response_params){
		    $attrStatement = $this->xml->createElement('saml:AttributeStatement');
		    $my_sp_attr = $response_params['Attributes'];
		    foreach ($my_sp_attr as $attr => $value) {
			   $attrs = $this->buildAttribute($attr,$value);
			   $attrStatement->appendChild($attrs);
			}
		    return $attrStatement;
		}
		
		function buildAttribute($attrName, $attrValue){
		    $attrs = $this->xml->createElement('saml:Attribute');
	  
		    $attrs->setAttribute('Name',$attrName);  
		    $attrs->setAttribute('NameFormat','urn:oasis:names:tc:SAML:2.0:attrname-format:basic');
	  
		    if(is_array($attrValue)){
			    foreach ($attrValue as $key => $val) {
				   $attrsValueElement = $this->xml->createElement('saml:AttributeValue',$val);
				   $attrs->appendChild($attrsValueElement);
			    }
		    }else{
			    $attrsValueElement = $this->xml->createElement('saml:AttributeValue',$attrValue);
			    $attrs->appendChild($attrsValueElement);
		    }
		    return $attrs;
        }
		
		function generateUniqueID($length) {
			$chars = "abcdef0123456789";
			$chars_len = strlen($chars);
			$uniqueID = "";
			for ($i = 0; $i < $length; $i++)
				$uniqueID .= substr($chars,rand(0,15),1);
			return 'a'.$uniqueID;
		}

		function signNode($private_key, $node, $subject,$response_params)
		{
			//Private KEY	
			$objKey = new XMLSecurityKey(XMLSecurityKey::RSA_SHA256,array( 'type' => 'private'));
			$objKey->loadKey($private_key, TRUE);
						
			//Sign the Assertion
			$objXMLSecDSig = new XMLSecurityDSig();
			$objXMLSecDSig->setCanonicalMethod(XMLSecurityDSig::EXC_C14N);

			$objXMLSecDSig->addReferenceList(array($node), XMLSecurityDSig::SHA1,
				array('http://www.w3.org/2000/09/xmldsig#enveloped-signature', XMLSecurityDSig::EXC_C14N),array('id_name'=>'ID','overwrite'=>false));
			$objXMLSecDSig->sign($objKey);
			$objXMLSecDSig->add509Cert($response_params['x509']);
			/*$certificate = $response_params['x509'];
			$certificate = str_replace( "-", "", $certificate );
			$certificate = str_replace( "BEGIN CERTIFICATE", "", $certificate );
			$certificate = str_replace( "END CERTIFICATE", "", $certificate );
			$certificate = str_replace( " ", "", $certificate );
			$objXMLSecDSig->add509Cert(trim($certificate),FALSE);*/			
			$objXMLSecDSig->insertSignature($node,$subject);
		}
		
	}